#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Apr 16 15:14:05 2020

@author:Irina Gorodetskaya
"""
# Script to compute SPC horizontal particle flux
# corrected for wind speed 

import pandas as pd
import numpy as np
from glob import glob
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import os
import argparse

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Process the horizontal particle flux data accounting for the wind")
    parser.add_argument("PF_path", type=str, help="Filepath to raw particle flux data")
    parser.add_argument("wind_path", type=str, help="Filepath to wind data")
    parser.add_argument("output_path", type=str, help="Output filepath")

    args = parser.parse_args()

# Time resolution:
freq='1min' 

# Download wind data corrected for the flow distortion:
wind=pd.read_csv(os.path.join(args.wind_path, "wind-observations-stbd-corrected-1min-legs0-4.csv"),
                 names=['date_time', 'latitude', 'longitude', 
                 'wind_from_direction_relative_to_platform', 'wind_speed_relative_to_platform',
                 'wind_from_direction', 'wind_speed',
                 'bowward_relative_wind', 'portward_relative_wind',
                 'eastward_wind', 'northward_wind'],
                 skiprows=[0],
                 low_memory=False)

# Tranforms strings from given format into python datetime format
wind['date_time']=pd.to_datetime(wind['date_time'], format="%Y-%m-%dT%H:%M:%S+00:00")

# DataFrame: absolute wind speed corrected for the flow distortion:
dfc=wind[['wind_speed']]
dfc.index=wind['date_time']
del wind

# Download uncorrected wind data:
wind=pd.read_csv(os.path.join(args.wind_path, "wind-observations-stbd-uncorrected-1min-legs0-4.csv"),
                 names=['date_time', 'latitude', 'longitude', 
                 'wind_from_direction_relative_to_platform', 'wind_speed_relative_to_platform',
                 'wind_from_direction', 'wind_speed',
                 'bowward_relative_wind', 'portward_relative_wind',
                 'eastward_wind', 'northward_wind'],
                 skiprows=[0],
                 low_memory=False)

# Tranforms strings from given format into python datetime format
wind['date_time']=pd.to_datetime(wind['date_time'], format="%Y-%m-%dT%H:%M:%S+00:00")

# DataFrame: relative wind speed (uncorrected)
dfu=wind[['wind_speed_relative_to_platform']]
dfu.index=wind['date_time']
del wind

# Combine both wind data into one dataframe:
df=dfc
df['WSR']=dfu['wind_speed_relative_to_platform']


# Read data with total horizontal particle flux (files by day)
names_HPF=sorted(glob(args.PF_path + 'SPC_HPF_' + freq + '*.csv'))

HPF_all=pd.DataFrame() #DataFrame with HPF data for all days

for name in names_HPF[1:]:
    
    #Load horizontal paricle flux data [#.m⁻².s⁻¹]
    HPF=pd.read_csv(name, index_col=[0])    
    #Tranforms index from given format into python datetime format
    HPF.index=pd.to_datetime(HPF.index, format="%Y-%m-%d %H:%M:%S")

    day=HPF.index[0]
    
    HPF['Horizontal particle flux [#.m-2.s-1]']=pd.to_numeric(HPF['Horizontal particle flux [#.m-2.s-1]'])

    #Merge wind data in one file for the entire period
    HPF_all=HPF_all.append(HPF)

    
# Resample wind and HPF series to have consistent time steps: 
df_mean = df.resample('1T', label='right', closed='right').mean()

# Select wind data only for theSPC time interval:
df_mean = df_mean[(df_mean.index > '2017-01-09 00:00:00') & (df_mean.index <= '2017-03-18 00:00:00')]    


HPFall_mean= HPF_all.resample('1T', label='right', closed='right').mean()

# Get particle concentration: divide HPF by relatve wind speed
       
# Particle concentration [#.m⁻³]
C=HPFall_mean['Horizontal particle flux [#.m-2.s-1]']/df_mean['WSR']  
    
# Corrected horizontal particle flux [#.m⁻².s⁻¹]
HPF_true=C*df_mean['wind_speed']
    
option='filtering'
    
HPF=HPF_true    
# Filter the outlier:
if option=='filtering':        
    for j in HPF.index:
        if (HPF.loc[j]>2e8):   
            HPF.loc[j]=np.nan        
              
# Save into .csv
HPF.to_csv(args.output_path + 'SPC_HPF_windtrue_' + freq + '.csv',
                header=['horizontal_particle_flux [#.m-2.s-1]'], index_label='date_time', date_format='%Y-%m-%dT%H:%M:%S+00:00',na_rep='NaN')

####################
# Plot of 1-min data
HPFplot=pd.read_csv(args.output_path + 'SPC_HPF_windtrue_' + freq + '.csv', index_col=[0])

HPFplot.index=pd.to_datetime(HPFplot.index, format="%Y-%m-%dT%H:%M:%S+00:00")
plt.plot(HPFplot.index,HPFplot['horizontal_particle_flux [#.m-2.s-1]'],label='Total particle flux')
plt.legend()
plt.xlabel('Date [UTC]')
plt.ylabel('Horizontal particle flux [# m-2 s-1]')
plt.title('1-min Horizontal particle flux [# m-2 s-1]')
plt.grid(True)

myFmt = mdates.DateFormatter('%Y-%m-%d')
plt.gca().xaxis.set_major_formatter(myFmt)
plt.gcf().autofmt_xdate()
plt.savefig(args.output_path + 'SPC_HPF_windtrue_1min.png', dpi=199) # Save figure
plt.close() # Close the current figure